__author__ = 'Paolo Prandoni'

import sys
import os
import codecs

WRAPPER = r"""
\documentclass{article}
\usepackage{pstricks,pst-eps,pst-3dplot}
\usepackage{dsptricks,dspfunctions,dspblocks}
@EXTRA_PACKAGES@
\begin{document}
\pagestyle{empty}
  \begin{TeXtoEPS}
    @PSFIG_CODE@
  \end{TeXtoEPS}
\end{document}
"""

# Some constants
FIGURES_DIRNAME  = 'ps'
RENDERED_DIRNAME = 'rendered'
LATEX_CMD = 'latex'
DVIPS_CMD = 'dvips'
GSVIEW_CMD = "C:\Program Files\Ghostgum\gsview\gsview64.exe"
EXTRA_PACKAGES = ''

# override constants from settings file
try:
    from pstf_settings import *
except ImportError:
    pass


def compile(path, infile, outfile, extra=''):

    i = os.path.join(path, infile + '.tex')

    try:
        with codecs.open(i, 'r', 'utf-8-sig') as f:
            code = f.read()
    except IOError:
        print 'cannot read %s' % i
        return None

    # make tex file
    if extra != '' and EXTRA_PACKAGES != '':
        extra = ','.join((extra, EXTRA_PACKAGES))
    elif EXTRA_PACKAGES != '':
        extra = EXTRA_PACKAGES
    if extra != '':
        extra = '\usepackage{%s}' % extra
    tex = WRAPPER.replace('@EXTRA_PACKAGES@', extra)
    tex = tex.replace('@PSFIG_CODE@', code)

    # make sure rendered folder exists
    rendered_path = os.path.join(path, RENDERED_DIRNAME)
    if not os.path.exists(rendered_path):
        os.makedirs(rendered_path)

    tex_file = os.path.join(rendered_path, outfile + '.tex')
    try:
        with codecs.open(tex_file, 'wb', 'utf-8') as f:
            f.write(tex)
    except IOError:
        print 'cannot write %s' % tex_file
        return None

    # make EPS
    dvi_file = os.path.join(rendered_path, outfile + '.dvi')
    eps_file = os.path.join(rendered_path, outfile + '.eps')

    here = os.getcwd()
    os.chdir(rendered_path)

    ret = os.system('%s -quiet %s' % (LATEX_CMD, tex_file))
    os.remove(tex_file)
    os.remove(os.path.join(rendered_path, outfile + '.aux'))
    os.remove(os.path.join(rendered_path, outfile + '.log'))

    ret += os.system('%s -q -E %s -o %s' % (DVIPS_CMD, dvi_file, eps_file))
    os.remove(dvi_file)
    os.chdir(here)

    # zero is no error
    if ret == 0:
        return eps_file
    else:
        return None


def render(infile, extra=''):
    print "rendering %s" % infile
    (path, filename) = os.path.split(os.path.abspath(infile))
    if filename.endswith('.tex'):
        filename = filename[:-4]
    return compile(path, filename, filename, extra)


def view(infile, extra=''):
    eps = render(infile, extra)
    if eps is not None:
        import subprocess
        subprocess.Popen([GSVIEW_CMD, eps])


def renderall(dir, extra=''):
    rootDir = dir
    for dirName, subdirList, fileList in os.walk(rootDir):
        if dirName.endswith(os.path.join('', FIGURES_DIRNAME)):
            for filename in fileList:
                if filename.endswith('.tex'):
                    render(os.path.join(dirName, filename), extra)


def usage(exe):
    (dummy, exename) = os.path.split(os.path.abspath(exe))
    print """
%s: pre-render pstricks code snippets to EPS figures.

Usage:
  %s render filename1 [filename2 ...]
     create EPS figures from the given source files

  %s view filename
     create EPS figure and launch viewer

  %s renderall dir
     search recursively from the given directory for all folders named \"%s\" and render all figures within

EPS figures are saved in a subdirectory named \"%s\".
You can append the option \"-use package1 package2 ...\" to use extra packages in the rendering
""" \
          % (exename, exename, exename, exename, FIGURES_DIRNAME, RENDERED_DIRNAME)
    exit()

if __name__ == "__main__":
    if len(sys.argv) < 3:
        usage(sys.argv[0])

    extra = ''
    for i in range(3, len(sys.argv)):
        if sys.argv[i] == '-use':
            extra = ','.join(sys.argv[i+1:])
            break

    if (sys.argv[1] == 'view'):
        view(sys.argv[2], extra)
    elif (sys.argv[1] == 'render'):
        for file in sys.argv[2:]:
            if file == '-use':
                break
            render(file, extra)
    elif (sys.argv[1] == 'renderall'):
        renderall(sys.argv[2], extra)
    else:
        usage(sys.argv[0])